import { asyncTransaction, transaction } from '../../../../../base/common/observableInternal/base.js';
import '../../../../../base/common/arrays.js';
import '../../../../../base/common/event.js';
import '../../../../../base/common/lifecycle.js';
import '../../../../../base/common/observableInternal/autorun.js';
import '../../../../../base/common/observableInternal/derived.js';
import '../../../../../base/common/cancellation.js';
import '../../../../../base/common/observableInternal/utils.js';
import { splitLines } from '../../../../../base/common/strings.js';
import { localize } from '../../../../../nls.js';
import { MenuId, Action2 } from '../../../../../platform/actions/common/actions.js';
import { IClipboardService } from '../../../../../platform/clipboard/common/clipboardService.service.js';
import { IConfigurationService } from '../../../../../platform/configuration/common/configuration.service.js';
import { ContextKeyExpr } from '../../../../../platform/contextkey/common/contextkey.js';
import { EditorAction } from '../../../../browser/editorExtensions.js';
import { EditorContextKeys } from '../../../../common/editorContextKeys.js';
import { Context } from '../../../suggest/browser/suggest.js';
import { showNextInlineSuggestionActionId, showPreviousInlineSuggestionActionId, inlineSuggestCommitId } from './commandIds.js';
import { InlineCompletionContextKeys } from './inlineCompletionContextKeys.js';
import { InlineCompletionsController } from './inlineCompletionsController.js';

class ShowNextInlineSuggestionAction extends EditorAction {
    static { this.ID = showNextInlineSuggestionActionId; }
    constructor() {
        super({
            id: ShowNextInlineSuggestionAction.ID,
            label: ( localize(5105, "Show Next Inline Suggestion")),
            alias: 'Show Next Inline Suggestion',
            precondition: ( (ContextKeyExpr.and(
                EditorContextKeys.writable,
                InlineCompletionContextKeys.inlineSuggestionVisible
            ))),
            kbOpts: {
                weight: 100,
                primary: 512  | 94 ,
            },
        });
    }
    async run(accessor, editor) {
        const controller = InlineCompletionsController.get(editor);
        controller?.model.get()?.next();
    }
}
class ShowPreviousInlineSuggestionAction extends EditorAction {
    static { this.ID = showPreviousInlineSuggestionActionId; }
    constructor() {
        super({
            id: ShowPreviousInlineSuggestionAction.ID,
            label: ( localize(5106, "Show Previous Inline Suggestion")),
            alias: 'Show Previous Inline Suggestion',
            precondition: ( (ContextKeyExpr.and(
                EditorContextKeys.writable,
                InlineCompletionContextKeys.inlineSuggestionVisible
            ))),
            kbOpts: {
                weight: 100,
                primary: 512  | 92 ,
            },
        });
    }
    async run(accessor, editor) {
        const controller = InlineCompletionsController.get(editor);
        controller?.model.get()?.previous();
    }
}
class TriggerInlineSuggestionAction extends EditorAction {
    constructor() {
        super({
            id: 'editor.action.inlineSuggest.trigger',
            label: ( localize(5107, "Trigger Inline Suggestion")),
            alias: 'Trigger Inline Suggestion',
            precondition: EditorContextKeys.writable
        });
    }
    async run(accessor, editor) {
        const controller = InlineCompletionsController.get(editor);
        await asyncTransaction(async (tx) => {
            await controller?.model.get()?.triggerExplicitly(tx);
            controller?.playAccessibilitySignal(tx);
        });
    }
}
class AcceptNextWordOfInlineCompletion extends EditorAction {
    constructor() {
        super({
            id: 'editor.action.inlineSuggest.acceptNextWord',
            label: ( localize(5108, "Accept Next Word Of Inline Suggestion")),
            alias: 'Accept Next Word Of Inline Suggestion',
            precondition: ( (ContextKeyExpr.and(
                EditorContextKeys.writable,
                InlineCompletionContextKeys.inlineSuggestionVisible
            ))),
            kbOpts: {
                weight: 100  + 1,
                primary: 2048  | 17 ,
                kbExpr: ( (ContextKeyExpr.and(
                    EditorContextKeys.writable,
                    InlineCompletionContextKeys.inlineSuggestionVisible
                ))),
            },
            menuOpts: [{
                    menuId: MenuId.InlineSuggestionToolbar,
                    title: ( localize(5109, 'Accept Word')),
                    group: 'primary',
                    order: 2,
                }],
        });
    }
    async run(accessor, editor) {
        const controller = InlineCompletionsController.get(editor);
        await controller?.model.get()?.acceptNextWord(controller.editor);
    }
}
class AcceptNextLineOfInlineCompletion extends EditorAction {
    constructor() {
        super({
            id: 'editor.action.inlineSuggest.acceptNextLine',
            label: ( localize(5110, "Accept Next Line Of Inline Suggestion")),
            alias: 'Accept Next Line Of Inline Suggestion',
            precondition: ( (ContextKeyExpr.and(
                EditorContextKeys.writable,
                InlineCompletionContextKeys.inlineSuggestionVisible
            ))),
            kbOpts: {
                weight: 100  + 1,
            },
            menuOpts: [{
                    menuId: MenuId.InlineSuggestionToolbar,
                    title: ( localize(5111, 'Accept Line')),
                    group: 'secondary',
                    order: 2,
                }],
        });
    }
    async run(accessor, editor) {
        const controller = InlineCompletionsController.get(editor);
        await controller?.model.get()?.acceptNextLine(controller.editor);
    }
}
class AcceptInlineCompletion extends EditorAction {
    constructor() {
        super({
            id: inlineSuggestCommitId,
            label: ( localize(5112, "Accept Inline Suggestion")),
            alias: 'Accept Inline Suggestion',
            precondition: ( (ContextKeyExpr.or(
                InlineCompletionContextKeys.inlineSuggestionVisible,
                InlineCompletionContextKeys.inlineEditVisible
            ))),
            menuOpts: [{
                    menuId: MenuId.InlineSuggestionToolbar,
                    title: ( localize(5113, "Accept")),
                    group: 'primary',
                    order: 1,
                }, {
                    menuId: MenuId.InlineEditsActions,
                    title: ( localize(5113, "Accept")),
                    group: 'primary',
                    order: 1,
                }],
            kbOpts: [
                {
                    primary: 2 ,
                    weight: 200,
                    kbExpr: ( (ContextKeyExpr.or( (ContextKeyExpr.and(
                        InlineCompletionContextKeys.inlineSuggestionVisible,
                         (EditorContextKeys.tabMovesFocus.toNegated()),
                         (Context.Visible.toNegated()),
                         (EditorContextKeys.hoverFocused.toNegated()),
                        InlineCompletionContextKeys.inlineSuggestionHasIndentationLessThanTabSize
                    )),  (ContextKeyExpr.and(
                        InlineCompletionContextKeys.inlineEditVisible,
                         (EditorContextKeys.tabMovesFocus.toNegated()),
                         (Context.Visible.toNegated()),
                         (EditorContextKeys.hoverFocused.toNegated()),
                        (InlineCompletionContextKeys.hasSelection.toNegated()),
                        InlineCompletionContextKeys.cursorAtInlineEdit
                    ))))),
                },
                {
                    primary: 2048  | 3 ,
                    weight: 200,
                    kbExpr: ( (ContextKeyExpr.and(
                        EditorContextKeys.editorTextFocus,
                        InlineCompletionContextKeys.inlineEditVisible,
                         (Context.Visible.toNegated()),
                         (EditorContextKeys.hoverFocused.toNegated()),
                         (EditorContextKeys.tabMovesFocus.toNegated())
                    ))),
                }
            ],
        });
    }
    async run(accessor, editor) {
        const controller = InlineCompletionsController.get(editor);
        if (controller) {
            controller.model.get()?.accept(controller.editor);
            controller.editor.focus();
        }
    }
}
class JumpToNextInlineEdit extends EditorAction {
    constructor() {
        super({
            id: 'editor.action.inlineSuggest.jump',
            label: ( localize(5114, "Jump to next inline edit")),
            alias: 'Jump to next inline edit',
            precondition: InlineCompletionContextKeys.inlineEditVisible,
            menuOpts: [{
                    menuId: MenuId.InlineEditsActions,
                    title: ( localize(5115, "Jump")),
                    group: 'primary',
                    order: 2,
                    when: ( (InlineCompletionContextKeys.cursorAtInlineEdit.toNegated())),
                }],
            kbOpts: {
                primary: 2 ,
                weight: 201,
                kbExpr: ( (ContextKeyExpr.and(
                    InlineCompletionContextKeys.inlineEditVisible,
                    (InlineCompletionContextKeys.hasSelection.toNegated()),
                     (EditorContextKeys.tabMovesFocus.toNegated()),
                     (Context.Visible.toNegated()),
                     (EditorContextKeys.hoverFocused.toNegated()),
                     (InlineCompletionContextKeys.cursorAtInlineEdit.toNegated())
                ))),
            }
        });
    }
    async run(accessor, editor) {
        const controller = InlineCompletionsController.get(editor);
        if (controller) {
            controller.jump();
        }
    }
}
class HideInlineCompletion extends EditorAction {
    static { this.ID = 'editor.action.inlineSuggest.hide'; }
    constructor() {
        super({
            id: HideInlineCompletion.ID,
            label: ( localize(5116, "Hide Inline Suggestion")),
            alias: 'Hide Inline Suggestion',
            precondition: ( (ContextKeyExpr.or(
                InlineCompletionContextKeys.inlineSuggestionVisible,
                InlineCompletionContextKeys.inlineEditVisible
            ))),
            kbOpts: {
                weight: 100,
                primary: 9 ,
            }
        });
    }
    async run(accessor, editor) {
        const controller = InlineCompletionsController.get(editor);
        transaction(tx => {
            controller?.model.get()?.stop(tx);
        });
    }
}
class ToggleAlwaysShowInlineSuggestionToolbar extends Action2 {
    static { this.ID = 'editor.action.inlineSuggest.toggleAlwaysShowToolbar'; }
    constructor() {
        super({
            id: ToggleAlwaysShowInlineSuggestionToolbar.ID,
            title: ( localize(5117, "Always Show Toolbar")),
            f1: false,
            precondition: undefined,
            menu: [{
                    id: MenuId.InlineSuggestionToolbar,
                    group: 'secondary',
                    order: 10,
                }],
            toggled: ( (ContextKeyExpr.equals('config.editor.inlineSuggest.showToolbar', 'always')))
        });
    }
    async run(accessor, editor) {
        const configService = accessor.get(IConfigurationService);
        const currentValue = configService.getValue('editor.inlineSuggest.showToolbar');
        const newValue = currentValue === 'always' ? 'onHover' : 'always';
        configService.updateValue('editor.inlineSuggest.showToolbar', newValue);
    }
}
class DevExtractReproSample extends EditorAction {
    constructor() {
        super({
            id: 'editor.action.inlineSuggest.dev.extractRepro',
            label: ( localize(5118, "Developer: Extract Inline Suggest State")),
            alias: 'Developer: Inline Suggest Extract Repro',
            precondition: InlineCompletionContextKeys.inlineEditVisible,
        });
    }
    async run(accessor, editor) {
        const clipboardService = accessor.get(IClipboardService);
        const controller = InlineCompletionsController.get(editor);
        const m = controller?.model.get();
        if (!m) {
            return;
        }
        const repro = m.extractReproSample();
        const inlineCompletionLines = splitLines(JSON.stringify({ inlineCompletion: repro.inlineCompletion }, null, 4));
        const json = ( (inlineCompletionLines.map(l => '// ' + l))).join('\n');
        const reproStr = `${repro.documentValue}\n\n// <json>\n${json}\n// </json>\n`;
        await clipboardService.writeText(reproStr);
        return { reproCase: reproStr };
    }
}

export { AcceptInlineCompletion, AcceptNextLineOfInlineCompletion, AcceptNextWordOfInlineCompletion, DevExtractReproSample, HideInlineCompletion, JumpToNextInlineEdit, ShowNextInlineSuggestionAction, ShowPreviousInlineSuggestionAction, ToggleAlwaysShowInlineSuggestionToolbar, TriggerInlineSuggestionAction };
