import { IStringDictionary } from '../../../base/common/collections.js';
import { ProcessItem } from '../../../base/common/processes.js';
import { UriComponents } from '../../../base/common/uri.js';

interface IMachineInfo {
    os: string;
    cpus?: string;
    memory: string;
    vmHint: string;
    linuxEnv?: ILinuxEnv;
}
interface ILinuxEnv {
    desktopSession?: string;
    xdgSessionDesktop?: string;
    xdgCurrentDesktop?: string;
    xdgSessionType?: string;
}
interface IDiagnosticInfo {
    machineInfo: IMachineInfo;
    workspaceMetadata?: IStringDictionary<WorkspaceStats>;
    processes?: ProcessItem;
}
interface IDiagnosticInfoOptions {
    includeProcesses?: boolean;
    folders?: UriComponents[];
    includeExtensions?: boolean;
}
interface WorkspaceStatItem {
    name: string;
    count: number;
}
interface WorkspaceStats {
    fileTypes: WorkspaceStatItem[];
    configFiles: WorkspaceStatItem[];
    fileCount: number;
    maxFilesReached: boolean;
    launchConfigFiles: WorkspaceStatItem[];
    totalScanTime: number;
    totalReaddirCount: number;
}

export type { IDiagnosticInfo, IDiagnosticInfoOptions, ILinuxEnv, IMachineInfo, WorkspaceStatItem, WorkspaceStats };
