import { IConfigurationService } from '../../../../platform/configuration/common/configuration.service.js';
import { IDialogService } from '../../../../platform/dialogs/common/dialogs.service.js';
import { IExtensionDescription } from '../../../../platform/extensions/common/extensions.js';
import { IFileService } from '../../../../platform/files/common/files.service.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { ILogService } from '../../../../platform/log/common/log.service.js';
import { INotificationService } from '../../../../platform/notification/common/notification.service.js';
import { IProductService } from '../../../../platform/product/common/productService.service.js';
import { ResolverResult } from '../../../../platform/remote/common/remoteAuthorityResolver.js';
import { IRemoteAuthorityResolverService } from '../../../../platform/remote/common/remoteAuthorityResolver.service.js';
import { IRemoteExtensionsScannerService } from '../../../../platform/remote/common/remoteExtensionsScanner.service.js';
import { ITelemetryService } from '../../../../platform/telemetry/common/telemetry.service.js';
import { IWorkspaceContextService } from '../../../../platform/workspace/common/workspace.service.js';
import { IWorkspaceTrustManagementService } from '../../../../platform/workspace/common/workspaceTrust.service.js';
import { IBrowserWorkbenchEnvironmentService } from '../../environment/browser/environmentService.service.js';
import { IWorkbenchExtensionEnablementService, IWorkbenchExtensionManagementService, IWebExtensionsScannerService } from '../../extensionManagement/common/extensionManagement.service.js';
import { IExtensionHostKindPicker } from '../common/extensionHostKind.js';
import { IExtensionService } from '../common/extensions.service.js';
import { AbstractExtensionService, IExtensionHostFactory, ResolvedExtensions } from '../common/abstractExtensionService.js';
import { IExtensionManifestPropertiesService } from '../common/extensionManifestPropertiesService.service.js';
import { ExtensionsProposedApi } from '../common/extensionsProposedApi.js';
import { ILifecycleService } from '../../lifecycle/common/lifecycle.service.js';
import { IRemoteAgentService } from '../../remote/common/remoteAgentService.service.js';
import { IRemoteExplorerService } from '../../remote/common/remoteExplorerService.service.js';
import { IUserDataInitializationService } from '../../userData/browser/userDataInit.service.js';
import { IUserDataProfileService } from '../../userDataProfile/common/userDataProfile.service.js';

declare class ExtensionService extends AbstractExtensionService implements IExtensionService {
    private readonly _browserEnvironmentService;
    private readonly _webExtensionsScannerService;
    private readonly _userDataInitializationService;
    private readonly _userDataProfileService;
    private readonly _workspaceTrustManagementService;
    private readonly _remoteExplorerService;
    constructor(extensionsProposedApi: ExtensionsProposedApi, extensionHostFactory: IExtensionHostFactory, extensionHostKindPicker: IExtensionHostKindPicker, instantiationService: IInstantiationService, notificationService: INotificationService, _browserEnvironmentService: IBrowserWorkbenchEnvironmentService, telemetryService: ITelemetryService, extensionEnablementService: IWorkbenchExtensionEnablementService, fileService: IFileService, productService: IProductService, extensionManagementService: IWorkbenchExtensionManagementService, contextService: IWorkspaceContextService, configurationService: IConfigurationService, extensionManifestPropertiesService: IExtensionManifestPropertiesService, _webExtensionsScannerService: IWebExtensionsScannerService, logService: ILogService, remoteAgentService: IRemoteAgentService, remoteExtensionsScannerService: IRemoteExtensionsScannerService, lifecycleService: ILifecycleService, remoteAuthorityResolverService: IRemoteAuthorityResolverService, _userDataInitializationService: IUserDataInitializationService, _userDataProfileService: IUserDataProfileService, _workspaceTrustManagementService: IWorkspaceTrustManagementService, _remoteExplorerService: IRemoteExplorerService, dialogService: IDialogService);
    private _initFetchFileSystem;
    protected _scanWebExtensions(): Promise<IExtensionDescription[]>;
    protected _resolveExtensionsDefault(): Promise<ResolvedExtensions>;
    protected _resolveExtensions(): Promise<ResolvedExtensions>;
    protected _onExtensionHostExit(code: number): Promise<void>;
    protected _resolveAuthority(remoteAuthority: string): Promise<ResolverResult>;
}

export { ExtensionService };
